      Subroutine IONRATE(ISURF, YPSURF, XLON, XLAT, XP, ZQ)

! Subroutine IONRATE calculate ionization rate (ZQ: ion-pairs/cm3s) for given 
! surface type, longitude (in degree), latitude (in degree), and pressure (mb).
!
! Written by Fangqun Yu and Gan Luo, SUNY-Albany, 2010 
! (yfq@asrc.cestm.albany.edu)
! 

      use cam_logfile,   only: iulog

      INTEGER   :: ISURF, L, K
      REAL*8    :: XLAT,XLON,XP, ZQ
      REAL*8    :: MAGLAT, XMAGLAT, YPR, YQSOIL,YPSURF
      REAL*8, SAVE  :: YQ(91,203)  ! Ionization lookup table
                      !YQ(1,1):    Q for maglat = 0,  p = 5 mb
                      !YQ(91,203): Q for maglat = 90, p = 1015 mb
                      !YQ(L,K):    Q for maglat = L-1,p = K*5 mb
      LOGICAL, SAVE :: FIRST = .TRUE.

!      write(iulog,*) "JIALEI-1"
!
! INPUT:
! ISURF: Surface type: 1 for land, 0 for ocean, ice, and snow
! YPSURF: Surface pressure (in mb)
! XLON: longitude (in degree, -180 - 180)
! XLAT: latitude (in degree, -90 - 90)
! XP: Grid box pressure (in mb, 5 mb - 1015 mb)
!
! OUTPUT:
! ZQ: Ionization rate (in ion-pairs/cm3s)
!
!
! Read in the ionization rate lookup table
!
      IF(FIRST) THEN
         CALL READIONRATE(YQ)
         FIRST = .FALSE.
      ENDIF
!      write(iulog,*) "JIALEI-2"

      
!
! Find the magnetic latitude based on (LON, LAT)

      CALL GEO2MAGLAT(XLAT,XLON,XMAGLAT)
      MAGLAT= abs(XMAGLAT) !  magnetic latitude in degree

      L = INT(MAGLAT+0.5) + 1
      K = INT(XP/5.+0.5)
      if(K.gt.203) K=203
      ZQ = YQ(L,K)   ! GCR ionization rate from the lookup table

      IF(ISURF.EQ.1) THEN  ! Contribution from radioactive material from soil
        CALL IONSOIL(XP,YPSURF,YQSOIL)
      ELSE
        YQSOIL = 0.
      ENDIF
!      WRITE(6,101)XLON,XLAT,XMAGLAT,XP, L, K,YQSOIL,ZQ

      ZQ = ZQ + YQSOIL
 101  FORMAT(4F7.1, I4, I4, 2F7.1)
      RETURN
      END

!------------------------------------------------------------------------------
      subroutine READIONRATE(YQ)
!
! Read pre-calculated GCR ionization rate lookup table
! The lookup table is generated based on the scheme given in
! Usoskin and Kovaltsov (2006).
! 
      INTEGER   :: L, K, KP
!
! YQ: ionization rate in ion-pairs/cm3s
! maglat is magnitude latitude in degree
!
      REAL*8 :: YQ(91,203) 
                      !YQ(1,1):    Q for maglat = 0,  p = 5 mb
                      !YQ(91,203): Q for maglat = 90, p = 1015 mb
                      !YQ(L,K):    Q for maglat = L-1,p = K*5 mb

      WRITE(6,*)"Read in the ionization rate lookup table"
      CLOSE(30)
      OPEN(30,file = '/glade/work/jialeiz/input_SOA/YIONRATE.txt')
      READ(30,*)   ! first line is magnetic latitude in degree
      DO K=1,203      ! KP is pressure in mb, YQ in ion-pairs/cm3s
         READ(30,100)KP,(YQ(L,K),L=1,91)
      ENDDO
 100  FORMAT(I5,91F6.2)
      CLOSE(30)
      RETURN
      END

!------------------------------------------------------------------------------
!
      subroutine IONSOIL(YPR,YPSURF,YQSOIL)

!******************************************************************************
! Calculate ionization rate (ion-pairs/cm3s) due to Gama rays, Radon (over land)
! Fangqun Yu, SUNY-Albany

      INTEGER, PARAMETER   :: MAXH = 21
      INTEGER   :: IH, K
      REAL*8    :: YPR, YPSURF,XH0, XH, YQ, YQGAMA, YQRADON, YQSOIL
      REAL*8    :: YH(MAXH), QGAMA(MAXH), QRADON(MAXH)
      REAL*8    :: MAGLAT

!
      DATA (YH(k),k=1,21)/0.0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0, ! in km
     &                    2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0,10.0,15.0/
      DATA (QGAMA(k),k=1,21)/4.5,1.25,0.21,0.0,0.0,0.0,0.0,0.0,0.0,
     &              0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0/
      DATA (QRADON(k),k=1,21)/3.5,3.24,3.0,2.65,2.43,2.19,1.84,1.36,
     &                        0.97, 0.74,0.56,0.13,
     &                        0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0/

! Get altitude (km) from pressure (YPR in mb) based on standard atmosphere
!
      XH0 = 44.3308 - 4.94654*(100.*YPR)**0.190264   ! convert press to alt (km)
! Surface height
      XHSURF = 44.3308 - 4.94654*(100.*YPSURF)**0.190264  

      XH = XH0 - XHSURF


      IF(XH.LT.5) THEN   ! over land, no gama and radon above 5 km
         IF(XH.LT.1) THEN
            IH = INT(XH*10.)+1
         ELSE
            IH = 10.+INT(XH)
         ENDIF

         IH=MIN(IH,20)
         IH=MAX(IH,1)

         YQGAMA=QGAMA(IH)+(XH-YH(IH))*(QGAMA(IH+1)-QGAMA(IH))
         YQRADON=QRADON(IH)+(XH-YH(IH))*(QRADON(IH+1)-QRADON(IH))
         YQSOIL = YQGAMA + YQRADON
!         WRITE(6,*)XH0,XHSURF, IH,YQGAMA,YQRADON
      ELSE 
         YQSOIL = 0.
      ENDIF

      end subroutine IONSOIL
!------------------------------------------------------------------------------

       subroutine GEO2MAGLAT(lat0,lon0,maglat)
!******************************************************************************
!  Subroutine GEO2MAGLAT finds magnetic latitude from geo latitude and
!  longitude (Fangqun Yu, SUNY-Albany)
!
! Input: latitude and longitude in degree
! Output: magnetic latitude in degree

       REAL*8     :: lat0, lon0
       REAL*8     :: yz,yp,yx,yy,lat,lon,maglat,PI,YD2R,YGLA,YGLO
       REAL*8     :: az,ap,ax,ay,dot,theta

       PI = 3.1415926
       YD2R = PI/180.
       
       YGLA = 80.*YD2R  !magnetic dipole north pole latitude
       YGLO = -110.*YD2R  !magnetic dipole north pole longitude

       lat = lat0 * YD2R
       lon = lon0 * YD2R

       yz = sin(YGLA)
       yp = cos(YGLA)
       yx = yp * cos(YGLO)
       yy = yp * sin(YGLO)
       
       az = sin(lat)
       ap = cos(lat)
       ax = ap * cos(lon)
       ay = ap * sin(lon)
       dot = ax*yx + ay*yy + az*yz
       theta = acos(dot)  !theta is the magnetic colatitude of the point a

       maglat = (PI/2.-theta)/YD2R
       
       end subroutine GEO2MAGLAT
!
!------------------------------------------------------------------------------
!------------------------------------------------------------------------------

