c=============================================================================
c
c $Id: cldfield.F,v 1.1 2003/05/16 22:22:45 akinorii Exp $
c
c FILE
c   cldfield.F
c
c ROUTINES
c   cldfield
c
c=============================================================================

c-----------------------------------------------------------------------------
c 
c ROUTINE
c   cldfield
c
c DESCRIPTION
c   This routine calculates cloud liquid water content and cloud fraction (
c   large scale and convective) based on parameterizations in literature.
c
c   Liquid water content (James Hack, Journal of Climate, 1998,p 1499)
c   Stratiform cloud fraction (Sundgvist, ?)
c   Cumulus cloud fraction (Xu and Krueger,1991)

c Author
c   Yan Feng (U of M) modified from Xiaohong's code, which is based on 
c   qk_update.F
c
c ARGUMENTS
c     lwccol      : cloud liquid water content in each grid box, g/m3
c     cldfra      : cloud fraction (unitless)
c     eff_rad     : cloud droplet effective radius (m)
c     zlev_asl    : height above sea level at grid edge(m)
c     latdeg      : latitude     (deg)
c     kel1        : temperature  (degK)
c     press3c     : atmospheric pressure at the center of each grid box (mb)
c     humidity    : specific humidity (g/kg) 
c     cldmas      : convective mass flux at t1 (kg/m2*s)
c     lwi_flags   : array of flags that indicate land, water, or ice
c
c-----------------------------------------------------------------------------
      subroutine cldfield
     & (lwccol, cldfra, eff_rad,
     &  zlev_asl, latdeg, kel1, press3c, 
     &  humidity, cldmas, lwi_flags)

      use h_imp_dims
      use h_imp_phys_constants
      implicit none

c#     include "imp_dims.h"
c#     include "imp_phys_constants.h"

c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: lwi_flags(i1_gl:i2_gl, ju1_gl: j2_gl)

      real*8  :: lwccol     (i1:i2, ju1:j2, k1:k2)
      real*8  :: cldfra     (i1:i2, ju1:j2, k1:k2)
      real*8  :: eff_rad    (i1:i2, ju1:j2, k1:k2)
      real*8  :: zlev_asl   (i1:i2, ju1:j2, k1-1:k2)
      real*8  :: latdeg     (ju1_gl:j2_gl) 
      real*8  :: kel1       (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: press3c    (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: humidity   (i1:i2,   ju1:j2,   k1:k2)
      real*8  :: cldmas     (i1:i2, ju1:j2, k1:k2)

c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: il, ij, ik 

c     Convective cloud fraction 
      real*8, allocatable :: ran_cloud(:,:,:)

c     Relative humidity, 0-1
      real*8, allocatable :: relh(:)

c     Large scale cloud fraction 
      real*8, allocatable :: max_cloud(:,:,:) 

c     Coefficients used in calculation of cloud fractions
      real*8, allocatable :: c0(:), c1(:), c2(:), rhc(:)

      real*8, allocatable :: cmfcol(:)

c     height (1D) above sea surface at grid box center
      real*8, allocatable :: zlay_asl_1d(:)

      Allocate (ran_cloud(i1:i2, ju1:j2, k1:k2))
      Allocate (max_cloud(i1:i2, ju1:j2, k1:k2))
      Allocate (relh(k1:k2))
      Allocate (c0(k1:k2))
      Allocate (c1(k1:k2))
      Allocate (c2(k1:k2))
      Allocate (rhc(k1:k2))
      Allocate (cmfcol(k1:k2))
      Allocate (zlay_asl_1d(k1:k2))

c     ----------------
c     Begin execution.
c     ----------------

      if (pr_diag) then
        Write (6,*) 'Cldfield called by ', loc_proc
      end if


c     ----------------------------------------------------------
c     Compute cloud drop effective radius from Kiehl, JGR, 1994.
c     ----------------------------------------------------------

c     ocean, sea ice, and land (kel1 < 243.15)
      eff_rad(:,:,:) = 1.0d-5
      
c     -----------------
c     do loop ij,il
c     -----------------

         do ij = ju1, j2
            do il = i1, i2

               do ik = k1, k2                  
                  if (lwi_flags(il,ij) == 2) then ! land
                     if ((kel1(il,ij,ik) >= 243.15d0) .and.
     &                    (kel1(il,ij,ik) <= 263.15d0)) then

                        eff_rad(il,ij,ik) = 5.0d-6 -
     &                       5.0d-6 * ((kel1(il,ij,ik) + ABS_ZERO +
     &                       10.d0) / 20.d0)
                        
                     else if (kel1(il,ij,ik) > 263.15d0) then
                        
                        eff_rad(il,ij,ik) = 5.0d-6
                        
                     end if                     
                  end if    
                  
c                  zlay_asl_1d(ik) =
c     & (zlev_asl(il, ij, ik) + zlev_asl(il,ij,ik-1))*0.5d0 

                  zlay_asl_1d(ik) =
     & -29.3d0*kel1(il,ij,ik)*
     & Log(press3c(il,ij,ik) / 1013.25d0)
                  zlay_asl_1d(ik)=max(zlay_asl_1d(ik), 0.0d0)

               end do

c     -------------------------------------------------------------------
c     Calculate cloud water from a parameterization of Kiehl found in
c     "Sensitivity of the Simulated Climate to a Diagnostic Formulation
c     for Cloud Liquid Water" by James Hack, Journal of Climate, Vol. 11,
c     July 1998, p 1499.
c     -------------------------------------------------------------------

               lwccol(il, ij, :) =
     &              0.18d0 * Exp (- zlay_asl_1d(:) /
     &              (1080.0d0 +
     &              (2000.0d0 * Cos (latdeg(ij)*RADPDEG) ** 2.d0)))


c     ----------------------------------------------------------------
c     Calculate convective and stratiform cloud fractions from a
c     parameterization of Xu found in "Evaluation of Cloudiness
c     Parameterizations Using a Cumulus Ensemble Model" by Kuan-Man Xu
c     and Steven Krueger, Monthly Weather Review, Vol. 119 p 342.
c     ----------------------------------------------------------------

c     --------------------------------------------------------------
c     First calculate relative humidity from Seinfeld (1986) p. 181.
c     The first  rh is the temperature dependent parameter a.
c     The second rh is the saturation vapor pressure of water.
c     The third  rh is the actual relative humidity as a fraction.
c     Then make sure rh is between 0 and 1.
c     --------------------------------------------------------------
            
               relh(:) = 1.0d0 - (373.15d0 / kel1(il,ij,:))
            
               relh(:) =
     &              1013.25d0 * Exp (13.3185d0 * relh(:)    -
     &              1.9760d0 * relh(:)**2 -
     &              0.6445d0 * relh(:)**3 -
     &              0.1299d0 * relh(:)**4)

               relh(:) =
     &              humidity(il,ij,:) * MWTAIR / (MWTH2O * GPKG) *
     &              press3c(il,ij,:) / relh(:)

c               relh(:) = Max (Min (relh(:), 1.0d0), 0.0d0)
c     Then make sure rh is between 0 and 0.95.
               relh(:) = Max (Min (relh(:), 0.95d0), 0.0d0)

c     -----------------------------------------------------------------------
c     Now set up the constants to be used in the stratiform parameterization.
c     -----------------------------------------------------------------------
!==============Guangxing Lin found zlay_asl_1d could larger than 15000 =

               where ((zlay_asl_1d(:) >= 15000.0d0)) 
                  c1 (:) =  0.012d0
                  c2 (:) =  0.321d0
                  rhc(:) =  0.445d0
               end where
!==============Guangxing Lin===========================
               where ((zlay_asl_1d(:) < 15000.0d0) .and.
     &              (zlay_asl_1d(:) >  5900.0d0))
                  c1 (:) =  0.012d0
                  c2 (:) =  0.321d0
                  rhc(:) =  0.445d0
               end where

               where ((zlay_asl_1d(:) <= 5900.0d0) .and.
     &              (zlay_asl_1d(:) >  2350.0d0))
                  c1 (:) = -0.102d0
                  c2 (:) =  7.473d0
                  rhc(:) =  0.715d0
               end where

               where  (zlay_asl_1d(:) <= 2350.0d0)
                 c1 (:) =  0.144d0
                 c2 (:) =  8.050d0
                 rhc(:) =  0.780d0
              end where

c     ----------------------------------------
c     Calculate the stratiform cloud fraction.
c     ----------------------------------------

              where (relh(:) > rhc(:) .and.
     &             zlay_asl_1d(:) < 15000.0d0)

c  Sundgvist's parameterization
                 ran_cloud(il,ij,:) =
     &             1.0 - sqrt(1.0 - (relh(:) - rhc(:))/(1.0 - rhc(:)))

                 ran_cloud(il,ij,:) = Max (ran_cloud(il,ij,:), 0.0d0)

              elsewhere

                 ran_cloud(il,ij,:) = 0.0d0

              end where

c     --------------------------------------------------------------------
c     Now set up the constants to be used in the cumulus parameterization.
c     --------------------------------------------------------------------
!==============Guangxing Lin found zlay_asl_1d could larger than 15000 =
               where ((zlay_asl_1d(:) >= 15000.0d0)) 
                 c0(:) = 0.0131d0
                 c1(:) = 0.0123d0
                 c2(:) = 0.0030d0
              end where
!==============Guangxing Lin  ==========================


              where ((zlay_asl_1d(:) < 15000.0d0) .and.
     &             (zlay_asl_1d(:) >  5900.0d0))
                 c0(:) = 0.0131d0
                 c1(:) = 0.0123d0
                 c2(:) = 0.0030d0
              end where

              where ((zlay_asl_1d(:) <= 5900.0d0) .and.
     &             (zlay_asl_1d(:) >  2350.0d0))
                 c0(:) = 0.0722d0
                 c1(:) = 0.0760d0
                 c2(:) = 0.0254d0
              end where

              where  (zlay_asl_1d(:) <= 2350.0d0)
                 c0(:) = 0.0337d0
                 c1(:) = 0.0453d0
                 c2(:) = 0.0237d0
              end where

c     -------------------------------------
c     Calculate the cumulus cloud fraction.
c     -------------------------------------

              cmfcol(:) = cldmas(il,ij,:) * 3600. * 9.8 / 100. ! mb/hr
cc or * 3600 * 1.0d-3 ??? (used in Impact model, qk_update.F)
c     cmfcol(:) = cldmas(il,ij,:) * 3600. * 1.0d-3

              where (cmfcol(:) > 0.01d0)

                 max_cloud(il,ij,:) = c0(:) +
     &                c1(:) *  Log10 (cmfcol(:)) +
     &                c2(:) * (Log10 (cmfcol(:)))**2

                 max_cloud(il,ij,:) = Max (max_cloud(il,ij,:), 0.0d0)

              elsewhere
                 
                 max_cloud(il,ij,:) = 0.0d0

              end where


c             -----------------------------------------------------------
c             Compute total cloud fraction ofr combined large-scale and 
c             convective clouds.
c             ----------------------------------------------------------
              cldfra(il,ij,:) = 
     &             1.0d0 - 
     &             (1.0d0 - max_cloud(il,ij,:))*
     &             (1.0d0 - ran_cloud(il,ij,:))
              
c     -----------------
c     end loop of ij,il
c     -----------------
           end do               !ij
        end do                  !il

        Deallocate (ran_cloud)
        Deallocate (max_cloud)
        Deallocate (relh)
        Deallocate (c0)
        Deallocate (c1)
        Deallocate (c2)
        Deallocate (rhc)
        Deallocate (cmfcol)
        Deallocate (zlay_asl_1d)
        return
        end
