           module h_rbchemarrays
           use h_rbchemmech
           implicit none
           public
c=============================================================================
c
c $Id: rbchemarrays.h,v 1.4 2002/05/02 15:50:22 akinorii Exp $
c
c CODE DEVELOPER
c   Sandy Sillman, U of M. (IMPACT version, 2007)
c
c FILE
c   rbchemarrays.h
c
c DESCRIPTION: variable declaration for allocated arrays
c  that contain input passed from main program to rb (quad) chem solver
c  (including rate constants, passed from rates to main solver)
c
c These arrays are separately declared and allocated in the entry program
c   (quadchem_solver.F in IMPACT).
c This INCLUDE file is included in subsequent subroutines.
c
c NO COMMON BLOCKS.

c ==========================================================================
c
c rbchemarrays.h   from  chemvars.EXT    April, 2007
c   INCLUDE and COMMON file 
c     for RADICAL BALANCE-BACK EULER solver for chemistry (quadchem)
c      (chemmain.f cheminit.f chemrates chemsolve.f, linslv.f, jval2.f)
c
c chemarrays.EXT must go with  chemmech.EXT 
c       with chemmech.EXT first - it contains indices.
c
c  This file includes declarations and common blocks
c   for INPUT-OUTPUT ARRAYS
c   which are passed to and  from the solver for photochemistry 
c   at each time step.
c
c IMPACT VERSION:  These ARRAYS were originally in chemvars.EXT.
c   They are separated out here.  
c   They will be ALLOCATED  in chemmain1.f and passed to subroutines.
c   Include this file in all subsequent subroutines. 
c
c  Divided into ESSENTIAL input/output 
c    and OPTIONAL output which is rarely used.
c
c  ADDED: rate constant arrays moved here.
c
c  Other INCLUDE files: 
c   chemmech.EXT =  variables for the chem. mechanism
c     that are set once and do not change as the mechanism is called.
c   chemlocal.EXT = Variables that are shared among the subroutines
c     in the chemistry solver, but not used elsewhere and not saved.
c
c Program written by Sanford Sillman
c History:
c  12/06. Program written by Sandy Sillman based on commq7.EXT
c  7/07   IMPACT version (.h), arrays separated into ichemarrays.h
c -------------------------------------------------------------
c      implicit none  -- IMPLICIT  is in boxvars.EXT, chemlocal.EXT
c      implicit real*8 (a-h,o-z)

c ESSENTIAL index parameters:   c_kvec, c_icdim, c_rdim
c   moved to CHEMMECH

c READ/WRITE AND VECTOR INDICES - moved to chemmech.EXT.
c
c BASIC INPUT ARRAYS:  CONCENTRATIONS, DEPOSITION 
c
c  - Note:  Allocate/deallocate these in chemmain1; declare and pass through.
c 
c c_xcin:       input  species  concentration, molec/cm3
c                  including emissions during the time step
c c_xcout:         final species concentrations, molec/cm3
c                  for gas species: sum of gas + linked aqueous,
c                   in gas units (molec/cm3).
c                  for aqueous species: M/liter.  
c c_xcav:       average species concentration during the time step
c                 (to be used for wet deposition: 
c                xcwdep(M/cm2) = xcav(M/L)*rainfr*alt(cm)*.01 (dm2/cm2)
c c_xcemit:     emissions during time step, molec/cm3
c               (only used in expo. decay solution, usually zero)
c c_xcwdep:     Wet deposition in solver - Not used for now
c         
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3
       real*8 ::  c_xcav( c_kvec,c_cdim)  ! concentr, mol/cm3
       real*8 ::  c_xcemit( c_kvec,c_cdim) ! emis, molec/cm3
c 
c Old common block - cut for allocated and passed-through arrays
c      common/CHEM_BASIC/c_xcin, c_xcout, c_xcav, c_xcemit

c OPTIONAL CHEM OUTPUT VALUES:    
c  May be used for subsequent analysis but not necessary.
c  OPTION: Move this from chemvars to chemlocal
c
c    Note:  Allocate.
c
c  c_rp(c_kvec,c_cdim)  Rate of production of species,molec/cm3/timestep
c  c_rl(c_kvec,c_cdim)  Rate of loss  of species ic, molec/cm3/timestep
c  c_rr(c_kvec,c_rdim)    Rate of reaction, molec/cm3/timestep

      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3

c Old common block - cut for allocated and passed-through arrays
c     common/CHEM_OPTIONAL/ c_rp, c_rl, c_rr

c CHEMICAL REACTION RATES
c   Note:  Allocate in chemmain1. or chemsolve1
c    (ratek rateh rateq rateqq in chemmain1 if rates called here)
c     (see also rate parameters in chemmech.EXT
c       and optional output in chemvars.EXT)

c  ratek     Rate constants (s-1 or cm3 s-1 or equiv. aqueous)
c    (c_rk     Parameters for rate constant calculation in chemmech)
c    (c_rr     Calculated rate in solver: ratek*C1*C2 in chemvars)
c  rateh     Henry's law coefficients Moles/liter-atm
c    (c_rkh    Parameters for Henry's law coefficients in chemmech)
c  rateq     Aqueous equilibrium coefficients Moles/liter
c    (c_rkq    Parameters for aqueous eq. coefficients in chemmech)
c  rateqq    Special equilibrium coefficients Moles/liter
c    (c_rkqq   Parameters for Special eq. coefficients in chemmech)
c  rhdif     Henry's law coeff. modified by droplet diffusion
c  egasaq    Gas-to-aqueous transfer coefficient, s-1
c              (Varies for reaction based on accom, molwt)
c              (This is either input or calculated internally)
c   (c_gasaq   Input gas-aq transfer coefficient in chemvars)
c
      real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
      real*8 :: rateh(c_kvec,c_hdim)  ! Henry's law constants
      real*8 :: rateq(c_kvec,c_aqdim)  ! Aqueous eq. constants
      real*8 :: rateqq(c_kvec,c_aqqdim) ! Special eq. constants
      real*8 :: rhdif(c_kvec,c_hdim) ! H. mod by diffusion
      real*8 :: egasaq(c_kvec,c_hdim) ! Gas-to-aq transfer s-1

c Cut common for Allocated and passed-through species
c     common/CHEMRATEK/ ratek, rateh, rateq, rateqq, rhdif, egasaq
c

c
c ----------------------------------------
c END OF chemarrays.EXT
c ----------------------------------------
c ================================================================
c
           end module h_rbchemarrays
