c=============================================================================
c
c $Id: rbchemrates.F,v 1.1 2002/02/20 15:38:36 akinorii Exp $
c
c CODE DEVELOPER
c   Original code from Sandy Sillman, U of M.
c   IMPACT quadchem modifications:  Akinori Ito.
c   2007 version for IMPACT: Sandy Sillman
c
c FILE
c   rbchemrates.F
c
c ORIGINAL FILE
c   chemrates.f
c
c=============================================================================
c ROUTINES:  rbchemrates, bod, ytn
c   (replaces Brates)
c   (bod and ytn are functions used in rate calculation)

c  chemrates.f   April, 2007
c  MODIFIED to include input for aqueous aerosol heterog. reactions, 063009
c
c  Programs to calculate chemical reaction rates and hv rates
c   for the RADICAL BALANCE-BACK EULER solution for photochemistry.
c
c ARGUMENTS
c
c   c_temp                 : temperature (degK)
c   c_dens                 : air density
c   c_h2ogas or watv       : water vapor
c   c_saersa or  sad_lbs    : surface area densities (cm^2/cm^3)
c   c_aqaersa or  saq_aer    : surface area densities (cm^2/cm^3) ADDED 063009
c   ratek      : reaction rate constants (units vary) (molec/cm-3)
c   rateh      : Henry's law coefficients (mole lit-1 atm-1)
c   rateh_temp : Henry's law coefficients (units?)
c   rateq      : Aqueous equilibrium constants (mole lit-1)
c   rateqq     : Special equilibrium constants (mole lit-1)
c
c ROUTINES:
c  rbchemrates: program to determine rates
c  function bod:  3-body reaction rate determination
c  function ytn:  alkyl nitrate yield function
c
c  Program  written by Sanford Sillman
c  ---------------------------------------------------------
c  Program history:
c    12/06 Initial program by Sandy Sillman based on boxchemv7.f
c    7/07  IMPACT version
c  ---------------------------------------------------------
c
c
      subroutine rbchemrates
     &   (c_temp, c_dens, c_h2ogas, c_h2oliq, c_saersa,
     &                               clfrac, c_aqaersa,  ! ADDED 063009
     &    ratek, rateh,rateh_temp, rateq, rateqq,
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include secondary organic aerosols
     &    s_m0, s_mwaver)
c ** End of modifications by Luis Olcese, UM
c ********************************************

c
c This establishes reaction rate constants
c  as a function of input rate parameters, temperature and density.
c
c Final rate constants, temperature and density are vectorized.
c
c Rates are based on k= A*exp(-B/temp) with A, B from REACTION.DAT.
c Special formats are included for special rate constants (3-body, etc.)
c ALK4, ALK7 stoichometries have been converted to special rates.
c
c ---------------------------------------------------------
c NOTE:  HVRATE CONSTANTS ARE MOVED TO SUBROUTINE BHVRATES.
c ---------------------------------------------------------
c
c Inputs:
c   chemistry reaction parameters (RK)
c   temperature
c   density
c   h2o, aerosol surface area
c
c Outputs:  rate constants (RATEK)
c
c Called by:  either boxmain (as part of chem. solution, or for info)
c             or by quadchem (as part of chemistry solution)
c   IMPACT:   Called by Update_Qk ( qk_update)
c
c Calls to:   None.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c  7/07  IMPACT version
c
c -------------------------------------------------------------------
c
c ---------------------------------------------------------
      use h_imp_dims
      use h_imp_control
      use h_rbchemmech
      use h_rbchemlocal

      IMPLICIT NONE

c#     include "imp_dims.h"
c#     include "imp_control.h"
c#     include "rbchemmech.h"
c#     include "rbchemlocal.h"

c GOT HERE

c  VARIABLES FROM rbchemlocarr.h:
c   (not allocated)
croy      double precision :: ratek(c_kvec,c_rdim)  ! Rate constants
croy      double precision :: rateh(c_kvec,c_rdim)  ! Henry's law constants
croy      double precision :: rateq(c_kvec,c_rdim)  ! Aqueous eq. constants
croy      double precision :: rateqq(c_kvec,c_rdim) ! Special eq. constants
      double precision :: ratek(c_kvec,num_qks)  ! Rate constants
      double precision :: rateh(c_kvec,num_qhs)  ! Henry's law constants
      double precision :: rateh_temp(c_kvec,num_qhs)  ! Henry's law constants
      double precision :: rateq(c_kvec,num_qaqs)  ! Aqueous eq. constants
      double precision :: rateqq(c_kvec,num_qaqqs) ! Special eq. constants

c  IMPACT-related values.
      double precision ::  c_h2ogas(c_kvec)  ! H2O abs humid molec/cm3
      double precision ::  c_temp(c_kvec)    ! temperature K
      double precision ::  c_dens(c_kvec)    ! density molec cm-3
      double precision ::  c_saersa(c_kvec)  ! s aero surf area cm2/cm3
      double precision ::  c_aqaersa(c_kvec)  ! aq aero surf area cm2/cm3
      double precision ::  c_h2oliq(c_kvec)

c------------Guangxing Lin 111809---------------------------

      double precision :: clfrac
      double precision :: ratektemp1,ratektemp2 

c------------Guangxing Lin 111809----------------------- 

c LOCAL VARIABLES
      double precision :: tempx   ! temperature K
      double precision :: denx    ! density, molec/cm3
      double precision :: bod     ! Function: 3-bodyreaction rate cm3-sec
      double precision :: ytn     ! Function: RNO3 yield from RO2+NO

c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include secondary organic aerosols
      real*8 :: s_vp, s_kpart, s_kpart_298
      real*8 :: s_mwaver, s_m0
c ** End of modifications by Luis Olcese, UM
c ********************************************

c KK = VECTORIZED LOOP VARIABLE, SET AT ONE
      kk=1
c
c pi (if not built-in special)
c AND "RU" PARAMETER FOR  molecular speed (Barth, personal comm)
c   (8.314e7 g-cm2/s2-mol-K)   (note: 8.314e0 kg-m2/s2-mol-K)
c  Moved to CHEMVAR.EXT as PARAMETERS
c     pii = 3.141592654
c      RUMOLEC = 8.314E7


c  SET ALL RATE CONSTANTS (including initial index values for HV rates,)

c --------------------------
c LOOP TO SET RATE CONSTANTS
c --------------------------
      do     j=1,c_nreac

c Default
       ratek(kk, j) = c_rk(1,j)

c Standard 2-body reaction format
       if(   c_nrk(j).eq.0) then
c        do  kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
c        enddo
       endif

c Standard 2-body reaction format with input rate at 298 K.
       if(   c_nrk(j).eq.-1) then
c        do  kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)
     *       *exp(-c_rk(2,j)*(1./c_temp(kk)-0.0033557047))
c        enddo
       endif

c Standard format *T/300
       if(   c_nrk(j).eq.-2) then
c        do kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
     *                 *(c_temp(kk)/300.)**c_rk(3,j)
c        enddo
       endif

c Standard 3-body reaction format - bod function
       if(   c_nrk(j).eq.-3) then
c        do  kk=1,c_kmax
           tempx = c_temp(kk)
           denx = c_dens(kk)
           ratek(kk ,j)=bod( c_rk(1,j), c_rk(2,j), c_rk(3,j),
     *                       c_rk(4,j), c_rk(5,j), tempx, denx )
c        enddo
       endif

c Combined 2-body and  3-body reaction format
       if(   c_nrk(j).eq.-4) then
c        do  kk=1,c_kmax
           tempx = c_temp(kk)
           denx = c_dens(kk)
           ratek(kk ,j)=bod( c_rk(3,j), c_rk(4,j), c_rk(5,j),
     *                       c_rk(6,j), c_rk(7,j), tempx, denx )
           ratek(kk ,j)= ratek(kk,j)
     *                    * c_rk(1,j)*exp(-c_rk(2,j)/tempx)

c        enddo
       endif

c 2-body reaction format with nitrate yield subtracted
       if(   c_nrk(j).eq.-5) then
c        do  kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
           ratek(kk, j) = ratek(kk, j )
     *       * (1.- ytn(c_rk(3,j), c_temp(kk ), c_dens(kk )) )
c        enddo
       endif

c 2-body reaction format for  nitrate yield
       if(   c_nrk(j).eq.-6) then
c        do  kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
           ratek(kk, j) = ratek(kk, j )
     *       * (    ytn(c_rk(3,j), c_temp(kk ), c_dens(kk )) )
c        enddo
       endif

c Density-dependent format A exp(-B/T)* (C + D*dens)/(E + F*dens)
c    for CO, GLYX+OH
       if(   c_nrk(j).eq.-7) then
c        do  kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
           ratek(kk,j) = ratek(kk,j)
     *      *(c_rk(3,j) + c_rk(4,j)*c_dens(kk))
     *      /(c_rk(5,j) + c_rk(6,j)*c_dens(kk))
c        enddo
       endif


c Combined format, interacting two-body reactions with density
c   (for MO2+MO2 and DMS)
c A exp(-B/T) / (1 + (C+E*dens)* exp(-D/T))
       if(   c_nrk(j).eq.-8) then
c        do  kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
           ratek(kk,j) = ratek(kk,j) / ( 1. +
     *              (c_rk(3,j) + c_rk(5,j)*c_dens(kk) )
     *                   *exp(-c_rk(4,j)/c_temp(kk))    )
c        enddo
       endif

c Special HO2+HO2 rate: (Aexp(-B/t) + C*dens*exp(-D/t)) *(1+ E exp(-F/T)*H2O)
c  from Chameides around 1996
       if(   c_nrk(j).eq.-9) then
c        do  kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
           ratek(kk,j) = ratek(kk,j) +
     *              c_rk(3,j)*exp(-c_rk(4,j)/c_temp(kk)) * c_dens(kk)
           ratek(kk,j) = ratek(kk,j) *  (1. +
     *        c_rk(3,j)*exp(-c_rk(4,j)/c_temp(kk)) * c_h2ogas(kk)  )

c        enddo
       endif


c Special HNO3+OH rate:
c   A exp(-B/t) + C exp(-D/t)*E exp(-F/t)/(Cexp(-D/T)+Eexp(-F/T))
c  from Evans, Fiore 2003
       if(   c_nrk(j).eq.-10) then
c        do  kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
           beta(kk)    = c_rk(3,j)*exp(-c_rk(4,j)/c_temp(kk))
     *                              *c_dens(kk)
           gamma(kk)   = c_rk(5,j)*exp(-c_rk(6,j)/c_temp(kk))

           ratek(kk,j) = ratek(kk,j) +
     *          beta(kk)*gamma(kk)/(beta(kk)+gamma(kk))

c        enddo
       endif

c Special C3H8 format (IUPAC 2002):  Aexp(-B/t)/(1+C*T/300^(D*exp(-E/t)))
       if(   c_nrk(j).eq.-11) then
c        do  kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
           beta(kk)    = c_rk(4,j)*exp(-c_rk(5,j)/c_temp(kk))

           ratek(kk,j) = ratek(kk,j) / (1. +
     *         c_rk(3,j)* (c_temp(kk)/300.)**beta(kk)   )

c        enddo
       endif

c Sum of two reactions (for ACET+OH, JPL 2002):  A exp(-B/T) + C exp(-D/T)
       if(   c_nrk(j).eq.-12) then
c        do  kk=1,c_kmax
           ratek(kk ,j)= c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
     *                   + c_rk(3,j)*exp(-c_rk(4,j)/c_temp(kk))
c        enddo
       endif

c------------Guangxing Lin ----------11-20-2010--------------------
c Special rate for partitioned RIO2:  RI2<->RIO2_3 -> products, rates k23b, k3p
c  where k3p is temperature dependent.
c Rate = direct k3p * base case RIO2_3/RIO2 * adj ratio (k23b/(k23b+fac*k3p)
c   = k1*k2/(k2+fac*k1)   or k1 * (1./(1+fac(k1/k2))
c  where k1 = direct k3p * base case RIO2_3/RIO2 partition
c  and   k2  = k23b * same RIO2_3/RIO2 partition
c   compare to -8, -11
       if(   c_nrk(j).eq.-14) then
c        do  kk=1,c_kmax
           gamma(kk)   = c_rk(1,j)*exp(-c_rk(2,j)/c_temp(kk))
           beta(kk)    = c_rk(3,j)*exp(-c_rk(4,j)/c_temp(kk))

           ratek(kk,j) = gamma(kk)
           if(beta(kk).gt.0.) then
             ratek(kk,j) = 1. + c_rk(5,j)*(gamma(kk)/beta(kk) )
             if(ratek(kk,j).gt.0.) then
               ratek(kk,j) = gamma(kk)/ratek(kk,j)
             else
               ratek(kk,j)=gamma(kk)
             endif
           else
             ratek(kk,j)=gamma(kk)
           endif

c        enddo
       endif

c------------Guangxing Lin ----------11-20-2010--------------------

c REACTION ON SULFATE AEROSOL SURFACES.  c_nrk(j) = -20
c   ratek = saersa*(gamma/4)* [8RT/(pi*MW)]^0.5 (Pandis and Seinfeld)
c   for saersa, aerosol surface area in cm2/cm3
c   gamma = reaction efficiency (input)
c   MW = molecular weight (input)
c
       if(   c_nrk(j).eq.-20) then
c        do  kk=1,c_kmax
           ratek(kk ,j)=  c_saersa(kk) * (0.25d0*c_rk(1,j))
     *       * (  (8.d0*RUMOLEC   * c_temp(kk))/(pii*c_rk(2,j))  )**0.5
c        enddo
       endif

c REACTION ON AQUEOUS AEROSOL SURFACES.  c_nrk(j) = -23
c   Adapted from sulfate aerosol reaction:  
c   ratek = saersa*(gamma/4)* [8RT/(pi*MW)]^0.5 (Pandis and Seinfeld)
c   for saersa, aerosol surface area in cm2/cm3
c   gamma = reaction efficiency (input)
c   MW = molecular weight (input)
c
c   e.g.  rk1,2 = gamma, MW =  2.900E-03     72. 
c      RUMOLEC = 8.314E7   --> sqrt=3e4.  -> rate=saersa*20 = saersa*gamma*7e3
c      saersa =    cm2/cm3 = 1e-6  -> rate 2e-4 for gamma = 2.9e-3
c      for partition: s_kbase = 1.0d-2  -> for aero=gas, gamma=1

c-------------Guangxing Lin 11-18-09- ------------------------------------  
c   To modify the formula to account for cloud fraction
   
       if(   c_nrk(j).eq.-23) then
c        do  kk=1,c_kmax
c           ratektemp1 =  c_aqaersa(kk) * (0.25d0*c_rk(1,j))
c     *       * (  (8.d0*RUMOLEC   * c_temp(kk))/(pii*c_rk(2,j))  )**0.5
c           write(6,*) 'gxlin-ratek1-  ', ratek(kk,j) 
c          if ( clfrac ==1.0) then
c             ratek(kk,j) = ratektemp1
c          else 
c             ratektemp2 = clfrac * Dexp(-ratektemp1 * tdt) + 1.0d0 - clfrac
c             ratek(kk, j)= (-1.0d0/tdt) * DLog(ratektemp2)
c          end if   
c        if(  abs(ratek(kk,j))>1.0 ) then
c             write(6,*) 'gxlin-ratek2-, clfrac  ', ratek(kk,j),clfrac
c             write(6,*)' ratektemp1,ratektemp2,tdt = ', ratektemp1,ratektemp2,tdt   
c           end if
       ratek(kk,j)= 0.
       if(clfrac.gt.0.) then
           ratek(kk,j) =  c_aqaersa(kk) * (0.25d0*c_rk(1,j))
     *       * (  (8.d0*RUMOLEC   * c_temp(kk))/(pii*c_rk(2,j))  )**0.5
         if (clfrac.lt. 1.0) then
           ratektemp1 = clfrac*Exp(-ratek(kk,j)*tdt)+1.0d0-clfrac
           if(ratektemp1.gt.0.) then
             ratektemp2= (-1.0d0/tdt) * Log(ratektemp1)
           endif
           if (ratektemp2.lt.ratek(kk,j) ) ratek(kk,j)= ratektemp2 
         endif
       endif 

c----------------Guangxing Lin 11-18-09-----------------------------  

c        enddo
       endif

c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include secondary organic aerosols

c
c Source of SOA partition coefficients:

c M. Kanakidou, et al., Organic aerosol and global climate modelling: a 
c review. Atmos. Chem. Phys., 5, 1053-1123,2005

c K. Tsigaridis and M. Kanakidou, Global modelling of secondary organic 
c aerosol in the troposphere: a sensitivity analysis. Atmos. Chem. Phys., 
c 3, 1849-1869, 2003

c  the component vapor pressure  c_pv:

c Marie Camredon and Bernard Aumont, Assessment of vapor pressure 
c estimation methods for secondary organic aerosol modleing. Atmospheric 
c Environment 40, 2105-2116, 2006

c SOA formation

c *** Rate gas -> aerosol - evaporative

       if(c_nrk(j).eq.-21) then

         ratek(kk ,j) = s_kbase

       endif


c *** Rate aerosol -> gas - evaporative

       if(c_nrk(j).eq.-22) then

c ***  This is another possible scheme for the dependence
c *** of the partition coefficient on the temperature.
c ***  To use it, uncomment these lines and comment the following
c *** ones, from s_vp.... to s_kpart

c         s_vp = 10 ** (
c     &    -
c     &    (86.d0 + 0.4d0 * c_rk(2,j) + 1421.d0  * c_rk(3,j)) *
c     &    (c_rk(1,j) - c_temp(kk)) /
c     &    (19.1d0 * c_temp(kk))
c     &    +
c     &    ((-90.d0 - 2.1d0 * c_rk(2,j)) / 19.1d0) *
c     &    (((c_rk(1,j) - c_temp(kk)) / c_temp(kk)) -
c     &    log(c_rk(1,j) / c_temp(kk))))

c         s_kpart = 8.21e-5 * c_temp(kk) / ( 1.e6 * s_mwaver * s_vp )

         s_vp = 10 ** (
     &    -
     &    (86.d0 + 0.4d0 * c_rk(2,j) + 1421.d0  * c_rk(3,j)) *
     &    (c_rk(1,j) - 298.d0) /
     &    (19.1d0 * 298.d0)
     &    +
     &    ((-90.d0 - 2.1d0 * c_rk(2,j)) / 19.1d0) *
     &    (((c_rk(1,j) - 298.d0) / 298.d0) -
     &    log(c_rk(1,j) / 298.d0)))

         s_kpart_298 = 8.21d-5 * 298.d0 / ( 1.d6 * s_mwaver * s_vp )

         s_kpart = s_kpart_298 * c_temp(kk) / 298.d0 * exp(
     &    s_dh * 1000.d0 / 8.314d0 * (1.d0 / c_temp(kk)
     &    - 1.d0 / 298.d0 ))

         ratek(kk ,j) = s_kbase / (s_kpart * s_m0)

       endif

c ** End of modifications by Luis Olcese, UM
c ********************************************

      enddo            ! END SET ALL RATE CONSTANTS LOOP
c --------------------------
c END - LOOP TO SET RATE CONSTANTS
c --------------------------


c ---------------------------
c HENRY'S LAW AND AQUEOUS RATE CONSTANTS
c ---------------------------

c HENRY'S LAW CONSTANT:
c UNIT CONVERSION FROM MOLES/LITER-ATMOS TO MOLES/LITER-molecules/cm2
c AND COMBINED WITH THE CONVERSION FACTOR FOR MOLES/LITER TO
c  MOLECULES PER CM2 (for use in AQUASOLVE).

c (NOTE:  Dimensionless Henry's law H' = HRT,  R=8.314E-02. L-atm/mol-K.
c   BARTH VALUE:  0.08206.
c   H' = HRT = Caq/Cg*L;  H" = HRTL.  L=acqua.  RT=avogadrl/atmos. OK).
c
c (with LIQ WATER 0.3e-6 gr/cm;  H = H'*1.248e5, H'=H*8E-6.)

c        (Replaced by parameter values in common)
c     atmos = 2.247E+19
c     avogadrl = 6.02E+20
c     rtcon= 0.08206

c  FUTURE CHANGE: ADD ALT. SPECIAL FORMULAS FOR HENRYS LAW CONSTANTS
c   AS NEEDED, CONTROLLED BY   c_nrkh()

      do 200 j=1,c_nreach
c      do 18051  kk=1,c_kmax
         rateh(kk ,j)= c_rkh(1,j)
     *             *exp(0.-c_rkh(2,j)*(1./c_temp(kk)-1./298.) )
         rateh_temp(kk ,j) = rateh(kk, j)
     *    *rtcon*c_temp(kk)
         rateh(kk ,j) = rateh(kk, j)
     *    *rtcon*c_temp(kk)*c_h2oliq(kk)
c
c (OLDER CONVERSION)
c        rateh(kk ,j) = rateh(kk, j)
c    *       *c_h2oliq(kk) * avogadrl/atmos
c
18051  continue
200   continue

c H+ AQUEOUS EQUILIBRIUM COEFFICIENTS (MOLES/LITER)
c
c  FUTURE CHANGE: ADD ALT. SPECIAL FORMULAS FOR EQUILIBRIUM COEFFICIENTS
c   AS NEEDED, CONTROLLED BY   c_nrkq()

      do 250 j=1,c_nreacq
c      do 18061  kk=1,c_kmax
         rateq(kk ,j)= c_rkq(1,j)
     *          *exp(0.-c_rkq(2,j)*(1./c_temp(kk)-1./298.) )
18061  continue
250   continue

c (CONVERSION:  AQUEOUS K1 IS MULTIPLIED BY HENRY'S LAW CONSTANT
c   AQUEOUS K2 IS MULTIPLIED BY K1KH, etc.
c DELETED )

c  SPECIAL AQUEOUS EQUILIBRIUM COEFFICIENTS (MOLES/LITER)
c  FUTURE CHANGE: ADD ALT. SPECIAL FORMULAS FOR SPECIAL EQ  COEFFICIENTS
c   AS NEEDED, CONTROLLED BY   c_nrkqq()


      do 255 j=1,c_nreaqq
c      do 18062  kk=1,c_kmax
         rateqq(kk ,j)= c_rkqq(1,j)
     *           *exp(0.-c_rkqq(2,j)*(1./c_temp(kk)-1./298.) )
18062  continue
255   continue

c ---------------------------
c END AQUEOUS RATE CONSTANTS
c ---------------------------

c END BRATES
2000    return
        end
c ---------------------------------------------------------------
c ----------------------------------------------------------

      FUNCTION bod(u,b,c,d,e, tempx, denx)
c
c  this is used to set termolecular rates.
c Need TEMP,DENS from COMMON.  CUT; passed on in argument.
c MODIFIED TO FIT WITH NASA 1992 FORM
c
c Inputs:
c  Parameters (From DeMore et al., 1997 (JPL), p.8)
c    b = ko(300) = Low pressure limit at 300 K.
c    c = exponent for temperature adjustment, -n in NASA.
c        (ko(T)=ko(300)*(T/300)**c)
c    d = kinf(300) = High pressure limit at 300 K.
c    e = exponent for temperature admustment, -m in NASA.
c      (kinf(T)=kinf(300)*(T/300)**e)
c    u = Fc, base for exponent in log-T-P adjustment, always 0.6 in JPL.
c  Meteorology
c    tempx = temperature
c    denx  = density, molec/cm3
c
c Output:
c    bod = reaction rate cm3-sec
c
c Called by:  chemrates
c Calls to:   none
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c  --------------------------------------------------------------
      IMPLICIT NONE

c
c LOCAL VARIABLES:  INPUT/OUTPUT
      double precision :: b       ! ko(300) = Low pressure limit at 300 K.
      double precision :: c       ! exponent for temperature, -n in JPL.
      double precision :: d       ! kinf(300), High pressure limit at 300 K
      double precision :: e       ! exponent for temperature, -m in JPL.
      double precision :: u       ! Fc, base for log-T-P adjustment (0.6)
      double precision :: tempx   ! temperature K
      double precision :: denx    ! density, molec/cm3
      double precision :: bod     ! Output reaction rate cm3-sec
c
c
c LOCAL VARIABLES:  INTERNAL
      double precision :: f1            ! Interim parameter
      double precision :: f2            ! Interim parameter
      double precision :: ee            ! Interim parameter
c
c
c     f1=b*(tempx**c)*denx
c     f2=f1/( d * (tempx**e))
c MODIFICATION TO TEMPX/300.
      f1=b*((tempx/300.)**c)*denx
      f2=f1/( d * ((tempx/300.)**e))

      ee= 1. / ( 1 + (log10(f2))**2. )
      bod = (f1/(1+f2) ) * u ** ee

      return
      end
c  --------------------------------------------------------------

      FUNCTION ytn(c, tempx, denx)
c
c  this sets the yield of alkyl nitrates from RO2+NO reactions.
c
c  Inputs:
c    c =  Number of carbon atoms in the RO2 (non-integer :: for lumped RO2)
c    tempx = temperature
c    denx  = density, molec/cm3
c
c  Output:
c    ytn = RNO3 yield (stoichiometry)
c
c Called by:  chemrates
c Calls to:   none
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c  --------------------------------------------------------------
      IMPLICIT NONE

c
c
c LOCAL VARIABLES:  INPUT/OUTPUT
      double precision :: c       ! Number of carbon atoms in RO2
      double precision :: tempx   ! temperature K
      double precision :: denx    ! density, molec/cm3
      double precision :: ytn     ! Output: RNO3 yield from RO2+NO
c
c LOCAL VARIABLES:  INTERNAL

      double precision :: x
      double precision :: y
      double precision :: z
      double precision :: par
      PARAMETER (par=4.3D-25)

      x=par*tempx*exp(1.08*c)*denx*(300./tempx)**5.05
      y=0.384*(300./tempx)**4.16
      z=1./ ( 1.+ (log10(x/y))**2. )

      ytn= ( x / (1.+ x/y ))*(0.467**z)

c 2009 CORRECTION: temporary correction for ytn >1 at t=210, c=7 (R7O2)
      if(ytn.gt.0.5) ytn = (0.25+ytn)/(1.+ytn)

      return
      end
c  --------------------------------------------------------------

c

c --------------------------------------------------------------
c END chemrates SUBROUTINES
c --------------------------------------------------------------

c =============================================================
c
c
