c-----------------------------------------------------------------------------
c $Id: tuv_lookup.F,v 1.2 2003/04/25 22:02:39 fengy Exp $
c 04/03/2003: add the option 3 to do linear average of optical depth

c TEST CHEM added for j-values - 
c   coordinate with  chem_update, qk_update and quadchem_solver
c
c DEBUG 807 WRITES ADDED - Sandy Sillman
c    (commented out)
c
c ROUTINE
c   Tuv_Lookup
c
c DESCRIPTION
c   This routine finds j-values used in Simpchem from lookup table made by
c   Sillman, 2003. The lookup table is built from Madronich's radiative 
c   transfer code (2000). The used version is solution of 8-stream discrete 
c   ordinate method. 

c   Input parameters are: date, zenith angle, O3 column burden (in DU), SO2 and
c   NO2 burden (in DU), surface albedo, aerosol column optical depth and single
c   scattering albedo, cloud optical depth and height above reference height 
c   and cloud otpical depth and height below reference height. 

c Author
c   Yan Feng (U of M)
c   01/16/2003

c Arguments
c     chemintv    : chemistry time step (s)
c     latdeg      : latitude     (deg)
c     rsec_jan1   : seconds from Jan. 1st (s)
c     press3c     : atmospheric pressure at the center of each grid box (mb)
c     pctm2       : psf, also CTM surface pressure at t1+tdt (mb)
c     ai          : pressure = (ai * pt) + (bi * psf), ai at zone interface
c     bi          : pressure = (ai * pt) + (bi * psf), bi at zone interface
c     pt          : defined in imp_control.h, do not need to pass      
c     kel1        : temperature  (degK)
c     const       : species concentration, known at zone centers (mixing ratio)
c     humidity    : specific humidity (g/kg)
c     mcor        : area of grid box (m^2)
c     mass        : total mass of the atmosphere within each grid box   (kg)
c     cmf1        : convective mass flux at t1 (kg/m2*s)
c     surf_alb_uv : bulk surface albedo (fraction 0-1)
c     grid_height : height of each grid box (m)
c     lwi_flags   : array of flags that indicate land, water, or ice
c     qjimp       : photolysis rate constants (s^-1)
c     do_trop_chem: do tropospheric chemistry
c
c-----------------------------------------------------------------------------

      subroutine Tuv_Lookup
     & (chemintv, latdeg, rsec_jan1, 
     &  press3c, pctm2, ai, bi, kel1, const,
     &  humidity, mcor, mass,
     &  cmf1, surf_alb_uv, grid_height, lwi_flags,
     &  qjimp, do_trop_chem, aero2d_opt, aero2d_ssa)

!      Guangxing Lin pass in aero2d_opt and aero2d_ssa

      use h_imp_time_constants
      use h_imp_phys_constants
      use h_imp_dims
      use h_imp_control
      use h_simpchem_cmq
      use h_setkin_par
      use h_imp_gen_io
      use h_rbchemmech

      use abortutils,    only: endrun
      implicit none

c#     include "imp_time_constants.h"
c#     include "imp_phys_constants.h"
c#     include "imp_dims.h"
c#     include "imp_control.h"
c#     include "simpchem_cmq.h"
c#     include "setkin_par.h"
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** From chemistry version
c#     include "imp_gen_io.h"
c ** End of modifications by Luis Olcese, UM
c ********************************************

c Added by Sandy Sillman, 2007 (to pass chem. mechanism to rate calc.)
c   NOTE, jparam has been added to one of the other include files.
c          (see jparam below)
c 
c#     include "rbchemmech.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      logical :: do_trop_chem(i1:i2, ju1:j2, k1:k2)

      integer :: lwi_flags(i1_gl:i2_gl, ju1_gl: j2_gl)

      real*8  :: chemintv
      real*8  :: latdeg     (ju1_gl:j2_gl)
      real*8  :: rsec_jan1
      real*8  :: kel1       (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: const      (i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: press3c    (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: pctm2      (ilo:ihi, julo:jhi)
      real*8  :: ai(k1-1:k2)
      real*8  :: bi(k1-1:k2)      
      real*8  :: mcor       (i1:i2,   ju1:j2)
      real*8  :: mass       (i1:i2, ju1:j2, k1:k2)
      real*8  :: qjimp      (i1:i2, ju1:j2, k1:k2, num_qjs)
      real*8  :: humidity   (i1:i2,   ju1:j2,   k1:k2)
      real*8  :: cmf1       (i1:i2, ju1:j2, k1:k2)
      real*8  :: surf_alb_uv(i1:i2, ju1:j2)
      real*8  :: grid_height(i1:i2, ju1:j2, k1:k2)      
      real*8  :: aero2d_opt(i1:i2,ju1:j2) !Guangxing Lin
      real*8  :: aero2d_ssa(i1:i2,ju1:j2) !Guangxing Lin      

c     -----------------------
c     Parameter declarations.
c     -----------------------

      integer, parameter :: COLMAX = 15
      real*8, parameter ::
     &  R_61  = (1.0d0 - (MWTH2O / MWTAIR)) / (MWTH2O / MWTAIR)

      real*8, parameter ::
     &  R29_3 = (GAS_CONST_J / MWTAIR * GPKG) / IMP_G
      
      real*8, parameter ::
     &  lwd = 1.0d+06,  ! liquid water densitry (g/m3)
     &  ext_eff = 2.0d0 ! extinction efficiency of a water droplet

c     ----------------------
c     Variable declarations.
c     ----------------------

      logical, save :: first = .true.
      integer :: il, ij, ik, ic, ihvintv, iq, i, iqj
      integer :: ncol           !total counted cases (only for cloudy sky)
      integer :: ncolx          !total cases
      integer :: icol
      integer :: iyr,imon,iday  !current year,month,day      
      real*8 :: sumcv           !total counted fractions
      real*8 :: stime_loc       !local time
      real*8 :: days
      real*8 :: cossza(1)       !cossine of zenith angle
      real*8 :: decl            !solar declination
      real*8 :: rdistsq         ! solar distance correction
      real*8, save :: lon_dummy(1) = 0.0d0     

c     ----------------------------------
c     Variables for look-up table inputs
c     ----------------------------------

      real*8  ::
     &  DUCONST  ! conversion constant for dobson unit

c     Ozone column, in Dobson Units.
      real*8, allocatable :: o3du(:,:)

c     cloud drop effective radius (m)
      real*8, allocatable :: eff_rad(:,:,:)

c     cloud liquid water content in each grid box, g/m3
      real*8, allocatable :: lwccol(:,:,:)

c     cloud fraction (unitless)
      real*8, allocatable :: cldfra(:,:,:)
      real*8, allocatable :: cldfra_1d(:)

c     cloud index
      real*8, allocatable :: cldinx(:,:)

c     fraction of each column possible case (only for cloudy sky)
      real*8, allocatable :: colfra(:)

c     cloud optical depth, unitless, 1-D array
      real*8, allocatable :: cld_opt(:)

c     height above sea surface (m) at grid box edge
      real*8, allocatable :: zlev_asl(:,:,:)
      real*8, allocatable :: zlev_asl_1d(:)
c     height above sea surface (m) at grid box center
      real*8, allocatable :: zlay_asl_1d(:)

        integer :: ierror(100),ierr_count
 
c Added by Sandy Sillman 8/29/07
c     ----------------------------------
c     Variables for look-up table output
c     ----------------------------------
c    output j-value array from jvalpro
c      real*8, allocatable ::      jval  (:)

        ierr_count = 0
         ierr_count = ierr_count+1

      Allocate (o3du(i1:i2, ju1:j2),stat=ierror(ierr_count))
           ierr_count = ierr_count+1
      Allocate (cld_opt(k1:k2),stat=ierror(ierr_count))
           ierr_count = ierr_count+1
      Allocate (eff_rad(i1:i2, ju1:j2, k1:k2),stat=ierror(ierr_count))
           ierr_count = ierr_count+1
      Allocate (lwccol(i1:i2, ju1:j2, k1:k2),stat=ierror(ierr_count))
           ierr_count = ierr_count+1
      Allocate (cldfra(i1:i2, ju1:j2, k1:k2),stat=ierror(ierr_count))
           ierr_count = ierr_count+1
      Allocate (cldfra_1d(k1:k2),stat=ierror(ierr_count))
           ierr_count = ierr_count+1
      Allocate (zlev_asl(i1:i2, ju1:j2, k1-1:k2),stat=ierror(ierr_count))
           ierr_count = ierr_count+1
      Allocate (zlev_asl_1d(k1-1:k2),stat=ierror(ierr_count))
           ierr_count = ierr_count+1
      Allocate (zlay_asl_1d(k1:k2),stat=ierror(ierr_count))
           ierr_count = ierr_count+1
      Allocate (cldinx(k1:k2, 1:COLMAX),stat=ierror(ierr_count))
           ierr_count = ierr_count+1
         Allocate (colfra(1:COLMAX),stat=ierror(ierr_count))
     
       if ( any( ierror(1:ierr_count) .ne. 0 ) ) then
           print*, ierror(1:ierr_count)
           call endrun("Error in allocation in tuv_lookup")

         end if


c Note:  parameters for Tuv code (jvalpro: nhv, hvmat, jarray, jval, etc)
c        are included in simpchem_cmq.h.  Use this, not c_nhv, etc.

c    output j-value array from jvalpro:  jparam, jval:
c      NOT NECESSARY.
c      ihv, nhv(22),  hvmat(22,40), hvmatb(22),jarray(80,510,56),
c      jparam(22), jval(56) -included in simpchem_cmq.h

c   Note, num_qjs must equal 56, so that jval matches jvalpro and qjimp.
c     Allocate jval(56)
c      Allocate jval(num_qjs)

      o3du = 0.0d0
      lwccol = 0.0d0
      zlev_asl = 0.0d0

c      jval = 0.0d0

c     ----------------
c     Begin execution.
c     ----------------

c DEBUG 807 WRITE: at START.   do_clear_sky, chem_opt(7), do_trop_chem()
c      write(6, 901) 
c      write(6,  * ) do_clear_sky, chem_opt, do_trop_chem(36,2,1)
c      write(6,*) ju1,j2, i1, i2, k1,k2 , nhvintv
c901   format(/,' DEBUG 807 in Tuv_lookup START: ',/,
c     & ' do_clear_sky, chem_opt(7), do_trop_chem(36,2,1) =')

      if (pr_diag) then
        Write (6,*) 'Tuv_Lookup called by ', loc_proc
        Write (6,*) 'Tuv_Lookup test 2' 
      end if
      
c     --------------------------------------------------------------------
c     Read in photolysis rate look up table generated by S. Sillman (2003)
c     Also read in aerosol optical properties (2D) here
c     --------------------------------------------------------------------
      
      if (first) then

         first = .false.
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** From chemistry version
c         if (chem_opt == 7 .or. chem_opt == 8) then
         if (chem_opt == 7 .or. chem_opt == 8 .or. chem_opt == 9) then
c ** End of modifications by Luis Olcese, UM
c ********************************************
           open(unit=999,file=lookup_infile_name,status='old')

c          ===========
           call readhv
c          ===========
c     &        (99,c_nhv,c_hvmat,c_hvmatb,c_jarray)
     &        (999,  nhv,  hvmat,  hvmatb,  jarray)
           close(999)

         end if
C
c--------Guangxing Lin-------------------------------
c------- use opt and ssa calculated in CAM5
c         Allocate (aero2d_opt (i1:i2, ju1:j2, 12))
c         Allocate (aero2d_ssa (i1:i2, ju1:j2, 12))
         
        
c-Guangxing Lin---temporarily set vaule ,need to update when put in CAM5
c         aero2d_opt=0.0d0
c         aero2d_ssa=4.0960724476811507D-033
cc        ===================
c         call Do_Lookup_Init
cc        ===================
c c    &        (aero2d_opt, aero2d_ssa, aop2d_infile_name)

      end if

c DEBUG 807 WRITE: at START, after first:
c      write(43,871)   nhv(1)
c      write(6,*) nhv(1), c_nhv(1)
c      write(43,872)    hvmat(1,1),   hvmat(1,2),   hvmat(1,3) ,
c!     &                    hvmat(1,4)
c!      write(43,873)   jarray(1,1,1),   jarray(1,1,2),   jarray(1,1,3),
c!     &   jarray(1,1,4)
871   format(/,'TUV:  NO C nhv =',i5)
872   format(/,'TUV:  NO C hvmat 1-4 for zenith angle =',/,
     &    (8(1pe10.3) ) )
873   format(' jarray (1,1,1)-(1,1,4) =',/,(8(1pe10.3)))

!      write(43,881) c_nhv(1)
!      write(43,882)  c_hvmat(1,1), c_hvmat(1,2), c_hvmat(1,3) ,
!     &                  c_hvmat(1,4)
!      write(43,883) c_jarray(1,1,1), c_jarray(1,1,2), c_jarray(1,1,3),
!     & c_jarray(1,1,4)

!881   format(/,'TUV: WITH C c_nhv =',i5)
!882   format(/,'TUV: WITH C c_hvmat 1-4 for zenith angle =',/,
!     &    (8(1pe10.3) ) )
!883   format(' c_jarray (1,1,1)-(1,1,4) =',/,(8(1pe10.3)))

c     --------------------------
c     Find out the current month
c     --------------------------
c     ====================      
      call Split_Time_Flds
c     ====================
     &     (nymd, iyr, imon, iday)      

c     -----------------------------------------------------------------------
c     Estimate ozone column, in Dobson units;
c     Assign surface albedo;
c     Assign aerosol optical depth and single scattering albedo;
c     Calculate grid box center height above sea level
c     ----------------------------------------------------------------------

      DUCONST = AVOGAD / (MWTAIR * 2.69d+16 * 10.0d0)

      do ij = ju1, j2
        do il = i1, i2
          do ik = k2, k1, -1
            if (do_synoz) then
              o3du(il, ij) = o3du(il, ij) +
     &        const(il, ij, ik, IO3S) * mass(il, ij, ik) *
     &        DUCONST / mcor(il, ij)
            else
              o3du(il, ij) = o3du(il, ij) +
     &        const(il, ij, ik, io3_num) * mass(il, ij, ik) *
     &        DUCONST / mcor(il, ij)
           end if

c DEBUG 907  - for DU ERROR
c                     if(  (il.eq.36.and.ij.eq. 2             ) .or.
c     &                    (il.eq.i1.and.ij.eq.ju1             )) then
!               write(43,811) il, ij, ik
!               write(43,*) do_synoz, IO3S, io3_num
!811             format(/,' DU IN TUV: il ij ik do, IO3S, io3num=',
!     &                          5i5)
!               write(43,812) o3du(il,ij),
!     &            const(il,ij,ik,IO3S), const(il,ij,ik, io3_num),
!     &            mass(il,ij,ik),
!     &             mcor(il,ij), DUCONST
!812            format(' o3du, constIO3S, constIO3num, mass,mcor,DUCON=',
!     &           /,(8(1pe10.3))   )

c                     endif

          end do


c          write(300+loc_proc,'(46e12.4)') 
c     &         (const(il,ij,ik,io3_num)*1.e9,ik=k1,k2)

c          write(400+loc_proc,'(46e12.4)')
c     &         (mass(il,ij,ik),ik=k1,k2)

c          write(200+loc_proc,'(2e12.4)') 
c     &         o3du(il,ij),mcor(il,ij)

          zlev_asl(il, ij, k1-1) = 
     &         R29_3 *  kel1(il, ij, k1) *
     &         (1.0d0 + R_61 * KGPG * humidity(il, ij, k1)) *
     &         Log (1013.25d0 / 
     &         (ai(k1-1)*pt + bi(k1-1)*pctm2(il, ij))) * KMPM

          zlev_asl(il, ij, k1-1) = Max (zlev_asl(il, ij, k1-1), 0.0d0)
          
          do ik = k1, k2

            zlev_asl(il, ij, ik) = zlev_asl(il, ij, ik-1) +
     &         R29_3 *  kel1(il, ij, ik) *
     &         (1.0d0 + R_61 * KGPG * humidity(il, ij, ik)) *
     &         Log ((ai(ik-1)*pt + bi(ik-1)*pctm2(il, ij))/ 
     &         (ai(ik)*pt + bi(ik)*pctm2(il, ij))) * KMPM

          end do        

        end do
      end do

c DEBUG 807 - AFTER INITIALIZATION.
c      write(6, 902)  ju1, j2, i1, i2, k1, k2
c902   format(/,' DEBUG 807 in Tuv_lookup AFTER INITIALIZATION: ',/,
c     & '   ju1,j2, i1, i2, k1, k2 =', 6i5)

c DEBUG 807 WRITE:
c      write(43,871)   nhv(1)
c      write(43,872)    hvmat(1,1),   hvmat(1,2),   hvmat(1,3) ,
c     &                    hvmat(1,4)
c      write(43,873)   jarray(1,1,1),   jarray(1,1,2),   jarray(1,1,3),
c     &   jarray(1,1,4)
c      write(43,881) c_nhv(1)
c      write(43,882)  c_hvmat(1,1), c_hvmat(1,2), c_hvmat(1,3) ,
c     &                  c_hvmat(1,4)
c      write(43,883) c_jarray(1,1,1), c_jarray(1,1,2), c_jarray(1,1,3),
c     & c_jarray(1,1,4)


!      qjimp(:,:,:,:)=0.0d0 !add this will cause memory corruption,
!      strange... Guangxing Lin
!======== Guangxing Lin=====debug ====
!      if(1.eq. 1) then
c      do_clear_sky = F
c     =====================
      if(do_clear_sky) then
c     =====================
         
        
c     =============
      else
c     =============

c DEBUG 807 - START OF CLOUD OVERLAP SCHEME
c      write(6, 904) cldlap_opt
c904   format(/,' DEBUG 807 in Tuv_lookup- before cldfield : ',/,
c     & ' cldlap_opt = ', i5)

c       =============
        call cldfield
c       =============
     &       (lwccol, cldfra, eff_rad,
     &       zlev_asl, latdeg, kel1, press3c, 
     &       humidity, cmf1, lwi_flags)         

c DEBUG 807 WRITE: at  start of cldlapopt 2
c      write(43,871)   nhv(1)
c      write(43,872)    hvmat(1,1),   hvmat(1,2),   hvmat(1,3) ,
c     &                    hvmat(1,4)
c      write(43,873)   jarray(1,1,1),   jarray(1,1,2),   jarray(1,1,3),
c     &   jarray(1,1,4)

c      write(43,881) c_nhv(1)
c      write(43,882)  c_hvmat(1,1), c_hvmat(1,2), c_hvmat(1,3) ,
c     &                  c_hvmat(1,4)
c      write(43,883) c_jarray(1,1,1), c_jarray(1,1,2), c_jarray(1,1,3),
c     & c_jarray(1,1,4)


        if(cldlap_opt == 1) then

          do ij = ju1, j2
            do il = i1, i2
               
              zlev_asl_1d(:) = zlev_asl(il, ij, :)
              do ik = k1, k2
                zlay_asl_1d(ik) =
     &                   (zlev_asl_1d(ik) + zlev_asl_1d(ik-1))*0.5d0
              enddo
              
              do ihv=1,22
                jparam(ihv)=0.0d0
              end do
                  
c             -----
c             DATE
              jparam(20) = nymd*1.d0
c             ----

              jparam(3) = o3du(il, ij)

c             --------------------------------------
c             Assume SO2, NO2 column in dobson units
              jparam(4) = 0.1
              jparam(5) = 0.1
c             --------------------------------------
              jparam(6) = aero2d_opt(il, ij)  !Guangxing Lin remove imon
              jparam(7) = aero2d_ssa(il, ij)  !Guangxing Lin remove imon

              jparam(8) = surf_alb_uv(il, ij)
                  
              do ihvintv = 1, nhvintv
c               ------------------
c               Solar zenith angle
                stime_loc = 
     &                   rsec_jan1+(ihvintv-0.5d0)*thvintv-1.0d0+
     &                   SECPDY * (il - 1.0d0) / i2_gl
                days = stime_loc / SECPDY

c               ============
                call Solrpos
c               ============
     &              (days, decl, rdistsq)               

c               ===========
                call Solrza
c               =========== 
     &               (days, decl, latdeg(ij), 
     &                lon_dummy, 1, cossza)

                jparam(1) = ACOSD (cossza(1))
c               --------------------------

                cldfra_1d(:)=cldfra(il, ij, :)

c               ===========
                call maxran
c               ===========
     &               (COLMAX, ivert+1, cldfra_1d, 
     &                ncol, cldinx, colfra, ncolx, sumcv)

                do icol = 1, ncol                        
                      
                  do ik = k1, k2                  
                    cld_opt(ik) = 0.0d0

                    if(cldinx(ik, icol).eq.1.) then
                      cld_opt(ik) = 
     &                         3.0d0 * grid_height(il, ij, ik)
     &                         * ext_eff * lwccol(il, ij, ik) / 
     &                         (4.0d0 * lwd * eff_rad(il, ij, ik))
                    else
                      cld_opt(ik) = 0.0d0
                    endif

                  end do

                  do ik = k1, k2
                    if (do_trop_chem(il,ij,ik)) then
                       jparam(2) = zlay_asl_1d(ik)

                       jparam(13) = kel1(il, ij, ik)
                                                
c                      ===========
                       call cldsum
c                      ===========
     &                      (ik, zlev_asl_1d, zlay_asl_1d,cld_opt,
     &                      jparam(9), jparam(10),
     &                      jparam(11),jparam(12))

c                      ============
                       call jvalpro
c                      ============
c     &                      (c_nhv, c_hvmat, c_hvmatb, c_jarray, 
     &                      (  nhv,   hvmat,   hvmatb,   jarray,
     &                       jparam, jval)


c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** From chemistry version
c                       if (chem_opt == 7) then
                       if (chem_opt == 7 .or. chem_opt == 9) then
c ** End of modifications by Luis Olcese, UM
c ********************************************
                         qjimp(il,ij,ik,:) =
     &                   qjimp(il,ij,ik,:) + jval(:) * colfra(icol)
                       else
                         qjimp(il,ij,ik,iO3_hv)=
     &                      qjimp(il,ij,ik,iO3_hv)
     &                      + jval(2) * colfra(icol)
                         qjimp(il,ij,ik,iNO2_hv) = 
     &                      qjimp(il,ij,ik,iNO2_hv) 
     &                      + jval(4) * colfra(icol)
                         qjimp(il,ij,ik,iHNO3_hv) = 
     &                      qjimp(il,ij,ik,iHNO3_hv) 
     &                      + jval(13) *colfra(icol)
                         qjimp(il,ij,ik,iH2O2_hv) =
     &                      qjimp(il,ij,ik,iH2O2_hv) 
     &                      + jval(11) * colfra(icol)
                         qjimp(il,ij,ik,iO3P3_hv) = 
     &                      qjimp(il,ij,ik,iO3P3_hv) 
     &                      + jval(3) * colfra(icol)
                         qjimp(il,ij,ik,iNO3P3_hv) = 
     &                      qjimp(il,ij,ik,iNO3P3_hv) 
     &                      + jval(6) * colfra(icol)
                         qjimp(il,ij,ik,iNO3O2_hv) =
     &                      qjimp(il,ij,ik,iNO3O2_hv) 
     &                      + jval(5) * colfra(icol)
                         qjimp(il,ij,ik,iN2O5_hv) = 
     &                      qjimp(il,ij,ik,iN2O5_hv) 
     &                      + jval(8) * colfra(icol)
                         qjimp(il,ij,ik,iN2O5P3_hv) = 
     &                      qjimp(il,ij,ik,iN2O5P3_hv) 
     &                      + jval(7) * colfra(icol)  
                         qjimp(il,ij,ik,iHCHO_hv) = 
     &                      qjimp(il,ij,ik,iHCHO_hv) 
     &                      + jval(15) * colfra(icol)
                         qjimp(il,ij,ik,iHCHOH2_hv) = 
     &                      qjimp(il,ij,ik,iHCHOH2_hv) 
     &                      + jval(16) * colfra(icol)        
                      end if

c TEST CHEM WRITE - coordinate with qk_update and quadchem_solver
c
          if(

c OPTION FOR NO TEST CHEM WRITE
     &         (il.eq.-305)

c OPTION FOR SPECIFIC GRIDS
c   NOTE - Coordinate this with TEST CHEM output
c     in qk_update.F, chem_update.F and quadchem_solver.F

c     &         (il.eq.36.and.ij.eq.2                           )
c     &   .or. (il.eq.59.and. ij.eq. 6                           )
c    &   .or. (il.eq.66.and. ij.eq. 9.and.(ik.eq.1.or.ik.eq.14) )

     &                                           )then
           write(6    , 201) il, ij, ik
           write(6    , 203)
     &          (jparam(i),i=1,12), jparam(13), jparam(20)
           write(6    ,205) jval(1), jval(2), jval(3), jval(4)
         endif   ! END TEST CHEM WRITE

c DEBUG 807 WRITE:  
c                     if(il.eq.36.and.ij.eq. 2             .and.
c     &                 (ihvintv.eq. 1.or.ihvintv.eq. nhvintv) )then
c                      write(43, 921) il, ij, ik,
c     &                  (jparam(i),i=1,12), jparam(13), jparam(20)
921    format(/,' DEBUG 807 in Tuv_lookup END (il, ij, ik= ',3i5,/,
     &                  ' jparam 1-13, 20 = ',/,
     &                   (8(1pe10.3))   )
c                    write(43,922) qjimp(il,ij,ik,2), qjimp(il,ij,ik,4)
922                    format('JO3, JNO2 = ', 2(1PE10.3)  )
c                      endif          !if(il.eq.36.and.ij.eq. 2 ) then


                    end if
                  end do      !ik

c                  if(il.eq.i1.and.ij.eq.ju1) then
c                     write(100+loc_proc,*) 'icol= ',icol
c                     do ihv=1,22
c                        write(100+loc_proc,*) 
c     &                       'jparam(',ihv,')= ',jparam(ihv)
c                     enddo
c                     write(100+loc_proc,*)
c                     do ik=k1,k2
c                        write(100+loc_proc,'(2e12.4)')
c     &                       cld_opt(ik),colfra(icol)
c                     enddo
c                  endif
                enddo          !icol
              end do            !ihvintv

              qjimp(il,ij,:,:) =
     &             qjimp(il,ij,:,:)/(nhvintv*1.0d0)

c     write(100+loc_proc,*) 'zen= ',jparam(1)
c     write(100+loc_proc,'(46e12.4)') 
c     &             (qjimp(il,ij,ik,iO3_hv),ik=k1,k2)              
            end do               !il
          end do                  !ij

c DEBUG 807 - START OF CLOUD OVERLAP SCHEME
c      write(43, 905) cldlap_opt
905   format(/,' DEBUG 807 in Tuv_lookup- after cldlap  1 : ',/,
     & ' cldlap_opt = ', i5)
            
        elseif(cldlap_opt == 2) then

c DEBUG 807 - START OF CLOUD OVERLAP SCHEME
c      write(43, 906) cldlap_opt
906   format(/,' DEBUG 807 in Tuv_lookup- start cldlapopt 2 ',/,
     & ' cldlap_opt = ', i5)

c      write(43,*) ju1,j2, i1, i2, k1,k2 , nhvintv

c DEBUG 807 WRITE: at  start of cldlapopt
c      write(43,871)   nhv(1)
c      write(43,872)    hvmat(1,1),   hvmat(1,2),   hvmat(1,3) ,
c     &                    hvmat(1,4)
c      write(43,873)   jarray(1,1,1),   jarray(1,1,2),   jarray(1,1,3),
c     &   jarray(1,1,4)

c      write(43,881) c_nhv(1)
c      write(43,882)  c_hvmat(1,1), c_hvmat(1,2), c_hvmat(1,3) ,
c     &                  c_hvmat(1,4)
c      write(43,883) c_jarray(1,1,1), c_jarray(1,1,2), c_jarray(1,1,3),
c     & c_jarray(1,1,4)

          do ij = ju1, j2
            do il = i1, i2
               
               zlev_asl_1d(:) = zlev_asl(il, ij, :)
               do ik = k1, k2
                  zlay_asl_1d(ik) =
     &                 (zlev_asl_1d(ik) + zlev_asl_1d(ik-1))*0.5d0
               enddo                 
                 
               do ihv=1,22
                  jparam(ihv)=0.0d0
               end do
                  
c              -----
c              DATE
               jparam(20) = nymd*1.d0
c              ----

               jparam(3) = o3du(il, ij)

c              --------------------------------------
c              Assume SO2, NO2 column in dobson units
               jparam(4) = 0.1
               jparam(5) = 0.1
c              --------------------------------------
               jparam(6) = aero2d_opt(il, ij) !Guangxing Lin
               jparam(7) = aero2d_ssa(il, ij) !Guangxing Lin

               jparam(8) = surf_alb_uv(il, ij)
                  
               do ihvintv = 1, nhvintv
c                 ----------------------------
c                 Solar zenith angle
                  stime_loc = 
     &                 rsec_jan1+(ihvintv-0.5d0)*thvintv-1.0d0+
     &                 SECPDY * (il - 1.0d0) / i2_gl
                  days = stime_loc / SECPDY

c                 ============
                  call Solrpos
c                 ============
     &                 (days, decl, rdistsq)               

c                 ===========
                  call Solrza
c                 =========== 
     &                 (days, decl, latdeg(ij), 
     &                 lon_dummy, 1, cossza)

                  jparam(1) = ACOSD (cossza(1))
c                 --------------------------

c DEBUG 807 WRITE: at BEFORE JVALPRO
c                     if(  (il.eq.36.and.ij.eq. 2             ) .or.
c     &                    (il.eq.i1.and.ij.eq.ju1             )) then

c      write(43,*) il,ij
c DEBUG 807 WRITE: at  start of cldlapopt 2
c      write(43,871)   nhv(1)
c      write(43,872)    hvmat(1,1),   hvmat(1,2),   hvmat(1,3) ,
c     &                    hvmat(1,4)
c      write(43,873)   jarray(1,1,1),   jarray(1,1,2),   jarray(1,1,3),
c     &   jarray(1,1,4)

c      write(43,881) c_nhv(1)
c      write(43,882)  c_hvmat(1,1), c_hvmat(1,2), c_hvmat(1,3) ,
c     &                  c_hvmat(1,4)
c      write(43,883) c_jarray(1,1,1), c_jarray(1,1,2), c_jarray(1,1,3),
c     & c_jarray(1,1,4)
c      endif
                  do ik = k1, k2                  
                        
                     cld_opt(ik) = 
     &                    3.0d0 * grid_height(il, ij, ik)
     &                    * ext_eff * lwccol(il, ij, ik) / 
     &                    (4.0d0 * lwd * eff_rad(il, ij, ik))

c                 ----------------------------------------------
c                 Approximate random overlapping (Briegleb, 1992)
c                 ----------------------------------------------
                     cld_opt(ik) = cld_opt(ik) * 
     &                    (cldfra(il, ij, ik) ** (3./2.))

                  end do
                
                  do ik = k1, k2

                     jparam(2) = zlay_asl_1d(ik)

                     jparam(13) = kel1(il, ij, ik)
                                          
c                    ===========
                     call cldsum
c                    ===========
     &                    (ik, zlev_asl_1d, zlay_asl_1d, cld_opt,
     &                    jparam(9), jparam(10),
     &                    jparam(11), jparam(12))

c DEBUG 807 WRITE: at BEFORE JVALPRO
c                     if(  (il.eq.36.and.ij.eq. 2.and.ik.le.10) .or.
c     &                    (il.eq.i1.and.ij.eq.ju1.and.ik.le.10)) then
c      write(43,*) il,ij,ik
c DEBUG 807 WRITE: at  start of cldlapopt 2
c      write(43,871)   nhv(1)
c      write(43,872)    hvmat(1,1),   hvmat(1,2),   hvmat(1,3) ,
c     &                    hvmat(1,4)
c      write(43,873)   jarray(1,1,1),   jarray(1,1,2),   jarray(1,1,3),
c     &   jarray(1,1,4)

c      write(43,881) c_nhv(1)
c      write(43,882)  c_hvmat(1,1), c_hvmat(1,2), c_hvmat(1,3) ,
c     &                  c_hvmat(1,4)
c      write(43,883) c_jarray(1,1,1), c_jarray(1,1,2), c_jarray(1,1,3),
c     & c_jarray(1,1,4)

c          end if
c                    ============
                     call jvalpro
c                    ============
c     &                    (c_nhv, c_hvmat, c_hvmatb, c_jarray,
     &                    (  nhv,   hvmat,   hvmatb,   jarray,
     &                     jparam, jval)

c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** From chemistry version
c                     if (chem_opt == 7) then
                     if (chem_opt == 7 .or. chem_opt == 9) then
c ** End of modifications by Luis Olcese, UM
c ********************************************
                     iqj = num_qjs
                      if(iqj.gt.56) iqj=56
                      do iq=1,iqj
                         qjimp(il,ij,ik,iq) =
     &                   qjimp(il,ij,ik,iq) + jval(iq)
                      enddo

c                       qjimp(il,ij,ik,:) =
c     &                 qjimp(il,ij,ik,:) + jval(:)
                     else
                       qjimp(il,ij,ik,iO3_hv)=
     &                    qjimp(il,ij,ik,iO3_hv) + jval(2)
                       qjimp(il,ij,ik,iNO2_hv) = 
     &                    qjimp(il,ij,ik,iNO2_hv) + jval(4)
                       qjimp(il,ij,ik,iHNO3_hv) = 
     &                    qjimp(il,ij,ik,iHNO3_hv) + jval(13)
                       qjimp(il,ij,ik,iH2O2_hv) =
     &                    qjimp(il,ij,ik,iH2O2_hv) + jval(11)
                       qjimp(il,ij,ik,iO3P3_hv) = 
     &                    qjimp(il,ij,ik,iO3P3_hv) + jval(3)
                       qjimp(il,ij,ik,iNO3P3_hv) = 
     &                    qjimp(il,ij,ik,iNO3P3_hv) + jval(6)
                       qjimp(il,ij,ik,iNO3O2_hv) =
     &                    qjimp(il,ij,ik,iNO3O2_hv) + jval(5)
                       qjimp(il,ij,ik,iN2O5_hv) = 
     &                    qjimp(il,ij,ik,iN2O5_hv) + jval(8)
                       qjimp(il,ij,ik,iN2O5P3_hv) = 
     &                    qjimp(il,ij,ik,iN2O5P3_hv) + jval(7)    
                       qjimp(il,ij,ik,iHCHO_hv) = 
     &                    qjimp(il,ij,ik,iHCHO_hv) + jval(15)
                       qjimp(il,ij,ik,iHCHOH2_hv) = 
     &                    qjimp(il,ij,ik,iHCHOH2_hv) + jval(16)          

                    end if

c GOT HERE - THIS IS THE RIGHT PLACE TO WRITE:  jparam and hv rates.
c


c TEST CHEM WRITE - coordinate with qk_update and quadchem_solver
c
          if(

c OPTION FOR NO TEST CHEM WRITE
     &         (il.eq.-305)

c OPTION FOR SPECIFIC GRIDS
c   NOTE - Coordinate this with TEST CHEM output  
c     in qk_update.F, chem_update.F and quadchem_solver.F

c     &         (il.eq.36.and.ij.eq.2                           )
c     &   .or. (il.eq.59.and. ij.eq. 6                           )
c    &   .or. (il.eq.66.and. ij.eq. 9.and.(ik.eq.1.or.ik.eq.14) )

     &                                           )then
           write(6    , 201) il, ij, ik
201        format(/'TEST CHEM in tuv_update:       il,ij,ik= ',/,3i3)
           write(6    , 203) 
     &          (jparam(i),i=1,12), jparam(13), jparam(20)
203        format(  '  jparam(1-13, 20) =',/,(8(1pe10.3)) )            
           write(6    ,205) jval(1), jval(2), jval(3), jval(4)
205       format(' jval 1-4 =  ',/,(5(1pe10.3)) )
         endif   ! END TEST CHEM WRITE


c 
c DEBUG 807 WRITE:  
c                    if( il.eq.36.and.ij.eq. 2.and.ik.le.10) then
c                     if(  (il.eq.36.and.ij.eq. 2.and.ik.le.10) .or. 
c     &                    (il.eq.i1.and.ij.eq.ju1.and.ik.le.10)) then
c                      write(43, 923) il, ij, ik,
c     &                  (jparam(i),i=1,12), jparam(13), jparam(20)
923    format(/,' DEBUG 807 in Tuv_lookup END 2: il, ij, ik= ',3i5,/,
     &                  ' jparam 1-13, 20 = ',/,
     &                   (8(1pe10.3))   )
c                      write(43,924) jparam(3), o3du(il, ij)
924                     format('jparam3, o3du(il, ij) =', 2(1pe10.3))
c                      write(43,*) ihvintv, cldlap_opt
c                      write(43,922) qjimp(il,ij,ik,2), qjimp(il,ij,ik,4)
c                      write(43,922) jval(2), jval(4)

c                      write(43,*) ihvintv, cldlap_opt
c                      write(43,922) qjimp(il,ij,ik,2), qjimp(il,ij,ik,4)
c                      write(43,922) jval(2), jval(4)
c      write(43,*) il,ij,ik
c DEBUG 807 WRITE: at  start of cldlapopt 2
c      write(43,871)   nhv(1)
c      write(43,872)    hvmat(1,1),   hvmat(1,2),   hvmat(1,3) ,
c     &                    hvmat(1,4)
c      write(43,873)   jarray(1,1,1),   jarray(1,1,2),   jarray(1,1,3),
c     &   jarray(1,1,4)

c      write(43,881) c_nhv(1)
c      write(43,882)  c_hvmat(1,1), c_hvmat(1,2), c_hvmat(1,3) ,
c     &                  c_hvmat(1,4)
c      write(43,883) c_jarray(1,1,1), c_jarray(1,1,2), c_jarray(1,1,3),
c     & c_jarray(1,1,4)

c      write(43,893) jval(1), jval(2), jval(3), jval(4)
893   format(/,' JVAL 1-4 =',/,(4(1pe10.3)) )

c                    endif          !if(il.eq.36.and.ij.eq. 2 ) then

                  end do        !ik
               end do           !ihvintv

               qjimp(il,ij,:,:) =
     &              qjimp(il,ij,:,:)/(nhvintv*1.0d0)
c               if(il.eq.i1.and.ij.eq.ju1) then
c                  do ik=k1,k2
c                     write(100+loc_proc,'(2e12.4)') 
c     &                    qjimp(il,ij,ik,iO3_hv),zlay_asl_1d(ik)
c                  enddo
c               endif
            end do              !il
          end do                 !ij

        elseif(cldlap_opt == 3) then
          do ij = ju1, j2
            do il = i1, i2
               
               zlev_asl_1d(:) = zlev_asl(il, ij, :)
               do ik = k1, k2
                  zlay_asl_1d(ik) =
     &                 (zlev_asl_1d(ik) + zlev_asl_1d(ik-1))*0.5d0
               enddo                 
                 
               do ihv=1,22
                  jparam(ihv)=0.0d0
               end do
                  
c              -----
c              DATE
               jparam(20) = nymd*1.d0
c              ----

               jparam(3) = o3du(il, ij)

c              --------------------------------------
c              Assume SO2, NO2 column in dobson units
               jparam(4) = 0.1
               jparam(5) = 0.1
c              --------------------------------------
               jparam(6) = aero2d_opt(il, ij) !Guangxing Lin
               jparam(7) = aero2d_ssa(il, ij) !Guangxing Lin

               jparam(8) = surf_alb_uv(il, ij)
                  
               do ihvintv = 1, nhvintv
c                 ----------------------------
c                 Solar zenith angle
                  stime_loc = 
     &                 rsec_jan1+(ihvintv-0.5d0)*thvintv-1.0d0+
     &                 SECPDY * (il - 1.0d0) / i2_gl
                  days = stime_loc / SECPDY

c                 ============
                  call Solrpos
c                 ============
     &                 (days, decl, rdistsq)               

c                 ===========
                  call Solrza
c                 =========== 
     &                 (days, decl, latdeg(ij), 
     &                 lon_dummy, 1, cossza)

                  jparam(1) = ACOSD (cossza(1))
c                 --------------------------

                  do ik = k1, k2                  
                        
                     cld_opt(ik) = 
     &                    3.0d0 * grid_height(il, ij, ik)
     &                    * ext_eff * lwccol(il, ij, ik) / 
     &                    (4.0d0 * lwd * eff_rad(il, ij, ik))

c                 ----------------------------------------------
c                 Linear average of optical depth (commonly used)
c                 ----------------------------------------------
                     cld_opt(ik) = cld_opt(ik) * cldfra(il, ij, ik)

                     if(kel1(il, ij, ik) <= 190.66d0) then
                       cld_opt(ik) = 0.0d0
                     end if

                  end do

c            cld_opt(:) = tau_cloud(il,ij,:)


                    qjimp(il,ij,:,(num_qjs-1)) =
     &               cld_opt(:)                

                  do ik = k1, k2
                       
                     jparam(2) = zlay_asl_1d(ik)

                     jparam(13) = kel1(il, ij, ik)
                                          
c                    ===========
                     call cldsum
c                    ===========
     &                    (ik, zlev_asl_1d, zlay_asl_1d, cld_opt,
     &                    jparam(9), jparam(10),
     &                    jparam(11), jparam(12))

c                    ============
                     call jvalpro
c                    ============
c     &                    (c_nhv, c_hvmat, c_hvmatb, c_jarray,
     &                   (  nhv,   hvmat,   hvmatb,   jarray,
     &                    jparam, jval)
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** From chemistry version
c                     if (chem_opt == 7) then
                     if (chem_opt == 7 .or. chem_opt == 9) then
c ** End of modifications by Luis Olcese, UM
c ********************************************
                       qjimp(il,ij,ik,:) =
     &                 qjimp(il,ij,ik,:) + jval(:)
                     else
                       qjimp(il,ij,ik,iO3_hv)=
     &                    qjimp(il,ij,ik,iO3_hv) + jval(2)
                       qjimp(il,ij,ik,iNO2_hv) = 
     &                    qjimp(il,ij,ik,iNO2_hv) + jval(4)
                       qjimp(il,ij,ik,iHNO3_hv) = 
     &                    qjimp(il,ij,ik,iHNO3_hv) + jval(13)
                       qjimp(il,ij,ik,iH2O2_hv) =
     &                    qjimp(il,ij,ik,iH2O2_hv) + jval(11)
                       qjimp(il,ij,ik,iO3P3_hv) = 
     &                    qjimp(il,ij,ik,iO3P3_hv) + jval(3)
                       qjimp(il,ij,ik,iNO3P3_hv) = 
     &                    qjimp(il,ij,ik,iNO3P3_hv) + jval(6)
                       qjimp(il,ij,ik,iNO3O2_hv) =
     &                    qjimp(il,ij,ik,iNO3O2_hv) + jval(5)
                       qjimp(il,ij,ik,iN2O5_hv) = 
     &                    qjimp(il,ij,ik,iN2O5_hv) + jval(8)
                       qjimp(il,ij,ik,iN2O5P3_hv) = 
     &                    qjimp(il,ij,ik,iN2O5P3_hv) + jval(7)    
                       qjimp(il,ij,ik,iHCHO_hv) = 
     &                    qjimp(il,ij,ik,iHCHO_hv) + jval(15)
                       qjimp(il,ij,ik,iHCHOH2_hv) = 
     &                    qjimp(il,ij,ik,iHCHOH2_hv) + jval(16)          
                    end if

c TEST CHEM WRITE - coordinate with qk_update and quadchem_solver
c
          if(

c OPTION FOR NO TEST CHEM WRITE
     &         (il.eq.-305)

c OPTION FOR SPECIFIC GRIDS
c   NOTE - Coordinate this with TEST CHEM output
c     in qk_update.F, chem_update.F and quadchem_solver.F

c     &         (il.eq.36.and.ij.eq.2                           )
c     &   .or. (il.eq.59.and. ij.eq. 6                           )
c    &   .or. (il.eq.66.and. ij.eq. 9.and.(ik.eq.1.or.ik.eq.14) )

     &                                           )then
           write(6    , 201) il, ij, ik
           write(6    , 203)
     &          (jparam(i),i=1,12), jparam(13), jparam(20)
           write(6    ,205) jval(1), jval(2), jval(3), jval(4)
         endif   ! END TEST CHEM WRITE

c DEBUG 807 WRITE:
c                     if(il.eq.36.and.ij.eq. 2.and.ik.eq.k1.and.
c     &                 (ihvintv.eq. 1.or.ihvintv.eq. nhvintv) )then
c                      write(43, 921)
c     &                 (jparam(i),i=1,12), jparam(13), jparam(20)
c                    write(43,922) qjimp(il,ij,ik,2), qjimp(il,ij,ik,4)
c                      endif          !if(il.eq.36.and.ij.eq. 2 ) then



                  end do        !ik
               end do           !ihvintv

               qjimp(il,ij,:,:) =
     &              qjimp(il,ij,:,:)/(nhvintv*1.0d0)
c               if(il.eq.i1.and.ij.eq.ju1) then
c                  do ik=k1,k2
c                     write(100+loc_proc,'(2e12.4)') 
c     &                    qjimp(il,ij,ik,iO3_hv),zlay_asl_1d(ik)
c                  enddo
c               endif
            end do              !il
          end do                 !ij
      
        endif                     !End of cloud overlap
         
c     ===========
      endif                     !End of clear or overcast
c     ===========

c DEBUG 807 - END 
c      write(43, 907) cldlap_opt
907   format(/,' DEBUG 807x in Tuv_lookup- END OF ROUTINE  : ',/,
     & ' cldlap_opt = ', i5)


c      write(200+loc_proc,*) 'iO3_hv = ',iO3_hv
c      do ihv=1,22         
c         write(200+loc_proc,*) 'jparam(',ihv,')= ',jparam(ihv)
c      enddo
c      write(200+loc_proc,*) 
c      do ik=k1,k2
c         write(200+loc_proc,'(2e12.4)')
c     & cld_opt(ik),cldfra(i2,j2,ik)
c      enddo
!======== Guangxing Lin=====debug ====
!      end if 

      Deallocate (o3du)
      Deallocate (cld_opt)
      Deallocate (eff_rad)
      Deallocate (lwccol)
      Deallocate (cldfra)
      Deallocate (cldfra_1d)
      Deallocate (zlev_asl)
      Deallocate (zlev_asl_1d)
      Deallocate (zlay_asl_1d)
      Deallocate (cldinx)
      Deallocate (colfra)

      return
      end

